import logging

from app import analytics, app, userevents
from data import model
from flask import request
from auth.auth_context import get_authenticated_user, get_validated_token, get_validated_oauth_token

logger = logging.getLogger(__name__)

def track_and_log(event_name, repo, **kwargs):
  repository = repo.name
  namespace = repo.namespace_user.username
  metadata = {
    'repo': repository,
    'namespace': namespace,
  }
  metadata.update(kwargs)

  analytics_id = 'anonymous'

  authenticated_oauth_token = get_validated_oauth_token()
  authenticated_user = get_authenticated_user()
  authenticated_token = get_validated_token() if not authenticated_user else None

  logger.debug('Logging the %s to Mixpanel and the log system', event_name)
  if authenticated_oauth_token:
    metadata['oauth_token_id'] = authenticated_oauth_token.id
    metadata['oauth_token_application_id'] = authenticated_oauth_token.application.client_id
    metadata['oauth_token_application'] = authenticated_oauth_token.application.name
    analytics_id = 'oauth:{0}'.format(authenticated_oauth_token.id)
  elif authenticated_user:
    metadata['username'] = authenticated_user.username
    analytics_id = authenticated_user.username
  elif authenticated_token:
    metadata['token'] = authenticated_token.friendly_name
    metadata['token_code'] = authenticated_token.code

    if authenticated_token.kind:
      metadata['token_type'] = authenticated_token.kind.name

    analytics_id = 'token:{0}'.format(authenticated_token.code)
  else:
    metadata['public'] = True
    analytics_id = 'anonymous'

  extra_params = {
    'repository': '%s/%s' % (namespace, repository),
    'user-agent': request.user_agent.string,
  }

  # Publish the user event (if applicable)
  logger.debug('Checking publishing %s to the user events system', event_name)
  if authenticated_user:
    logger.debug('Publishing %s to the user events system', event_name)
    user_event_data = {
      'action': event_name,
      'repository': repository,
      'namespace': namespace
    }

    event = userevents.get_event(authenticated_user.username)
    event.publish_event_data('docker-cli', user_event_data)

  # Save the action to mixpanel.
  logger.debug('Logging the %s to Mixpanel', event_name)
  analytics.track(analytics_id, event_name, extra_params)

  # Log the action to the database.
  logger.debug('Logging the %s to logs system', event_name)
  model.log.log_action(event_name, namespace, performer=authenticated_user, ip=request.remote_addr,
                       metadata=metadata, repository=repo)

  logger.debug('Track and log of %s complete', event_name)
