import logging

from app import image_diff_queue
from data import model
from endpoints.v1.registry import process_image_changes
from workers.worker import Worker


logger = logging.getLogger(__name__)


class DiffsWorker(Worker):
  def process_queue_item(self, job_details):
    image_id = job_details['image_id']
    repository = job_details['repository']

    # TODO switch to the namespace_user_id branch only once exisiting jobs have all gone through
    if 'namespace_user_id' in job_details:
      namespace = model.get_namespace_by_user_id(job_details['namespace_user_id'])
    else:
      namespace = job_details['namespace']

    try:
      process_image_changes(namespace, repository, image_id)
    except model.DataModelException:
      # This exception is unrecoverable, and the item should continue and be
      # marked as complete.
      msg = ('Image does not exist in database \'%s\' for repo \'%s/\'%s\'' %
             (image_id, namespace, repository))
      logger.warning(msg)

    return True

if __name__ == "__main__":
  logging.config.fileConfig('conf/logging.conf', disable_existing_loggers=False)

  worker = DiffsWorker(image_diff_queue)
  worker.start()
